﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml;
using Ionic.Zip;
using TPM.Properties;

//using System.IO.Pipes;

namespace TPM
{
	/// <summary>
	/// IO Class dla dzialan na plikach oraz metody pomocnicze
	/// </summary>
	public partial class IOClass : Form
	{
		#region PROP

		//zmienna sluzaca do zliczenia wszystkich dostepnych plikow graficznych
		private const string PathToMC = @"%appdata%.minecraft\texturepacks";

		// trzy zmienne do okreslenia sciezek dostepu dla katalogu do grafik programu

		private Button _button1;
		private BackgroundWorker _bwDepacking;
		private BackgroundWorker _bwPack;
		private Label _label1;
		private int _multiplier = 16;
		private Form _orginalform;
		private string _pathtoconfig = @"PathesToFiles.xml";
		private ProgressBar _progressBar1;
		private string _selectedFileNameDST;
		private string _selectedFileNameSRC;
		private ZipFile _zip;
		public int FoundFiles { get; set; }

		private string Patchtmp { get; set; }

		private string Pathtoreturn { get; set; }

		private string TempDST { get; set; }

		private string TempSRC { get; set; }
		private string TempTpm { get; set; }
		private string SavePath { get; set; }

		public string SelectedFileNameSRC
		{
			get
			{
				if (_selectedFileNameSRC == null)
				{
					return "NO IMAGE";
				}
				return _selectedFileNameSRC;
			}
			set { _selectedFileNameSRC = value; }
		}

		public string SelectedFileNameDST
		{
			get
			{
				if (_selectedFileNameDST == null)
				{
					return "NO IMAGE";
				}
				return _selectedFileNameDST;
			}
			set { _selectedFileNameDST = value; }
		}

		#endregion

		/// <summary>
		/// Konstruktor klasy IO
		/// </summary>
		/// <param name="sendedform"></param>
		public IOClass(Form sendedform)
		{
			InitializeComponent();
			_orginalform = sendedform;
			TempTpm = Path.GetTempPath() + "TMP";
			TempSRC = TempTpm + "\\src\\";
			TempDST = TempTpm + "\\dst\\";
		}


		/// <summary>
		/// Metoda odczytuyje jakie powinny byc pliki z "PathesToFiles.xml i sprawdza czy takie
		/// pliki istnieja na dysku w katalogu temp/tmp/src i dst
		/// </summary>
		/// <param name="PathesListSRC">The pathes list SRC.</param>
		/// <param name="PathesListDST">The pathes list DST.</param>
		/// <param name="_incomingwhat">The _incomingwhat.</param>
		public void CzySaTakiePliki(ref List<string> PathesListSRC, ref List<string> PathesListDST, string _incomingwhat)
		{
			FoundFiles = 0;
			var _xml = new XmlDocument();
			_xml.Load(_pathtoconfig);
			XmlNodeList _node = _xml.GetElementsByTagName("TPM"); //szukaj childnody dla glowej galezi TPM
			int _childnodes = _node.Item(0).ChildNodes.Count; //odczyt ilosci childnodow dla glownej galezi
			if (_incomingwhat == "src") //czysci tablicze src
			{
				PathesListSRC.Clear();
			}
			if (_incomingwhat == "dst") // czysci tablice dst
			{
				PathesListDST.Clear();
			}
			for (int i = 0; i < _childnodes; i++) //MAIN LOOP, petla dla znalezionych nodow
			{
				if (_incomingwhat == "src")
				{
					Patchtmp = TempSRC + _node.Item(0).ChildNodes.Item(i).Attributes.GetNamedItem("path").InnerText;
					;
				}
				if (_incomingwhat == "dst")
				{
					Patchtmp = TempDST + _node.Item(0).ChildNodes.Item(i).Attributes.GetNamedItem("path").InnerText;
				}

				for (int a = 0; a < _node.Item(0).ChildNodes.Item(i).ChildNodes.Count; a++) //petla dla tagow noda
				{
					string _file = _node.Item(0).ChildNodes.Item(i).ChildNodes.Item(a).InnerText; //pobiera jaki plik z xml-a
					string _fullpath = Patchtmp + _file; //skleja sciezke z plikiem
					if (File.Exists(_fullpath))
					{
						if (_incomingwhat == "src")
						{
							PathesListSRC.Add(_fullpath);
						}
						if (_incomingwhat == "dst")
						{
							PathesListDST.Add(_fullpath);
						}
						FoundFiles++;
					}
				}
			}
		}

		public int FindMultilplierInXMLFile(string _whatToFind)
		{
			_multiplier = 1;
			var _xml = new XmlDocument();
			_xml.Load(_pathtoconfig);
			XmlNodeList _node = _xml.GetElementsByTagName("TPM"); //szukaj childnody dla glowej galezi TPM
			int _childnodes = _node.Item(0).ChildNodes.Count; //odczyt ilosci childnodow dla glownej galezi


			for (int i = 0; i < _childnodes; i++) //MAIN LOOP, petla dla znalezionych nodow
			{
				for (int a = 0; a < _node.Item(0).ChildNodes.Item(i).ChildNodes.Count; a++) //petla dla tagow noda
				{
					string nodeValue = _node.Item(0).ChildNodes.Item(i).ChildNodes.Item(a).InnerText; //pobiera jaki plik z xml-a
					if (nodeValue == _whatToFind)
					{
						_multiplier =
							Convert.ToInt32(
								_node.Item(0).ChildNodes.Item(i).ChildNodes.Item(a).Attributes.GetNamedItem("multiplier").InnerText);
					}
				}
			}
			return _multiplier;
		}


		public bool LoadZip(string where)
		{
			bool wynik = false;
			string _where = where;
			PrepareDirs(_where);
			using (var okienko = new OpenFileDialog())
			{
				okienko.Filter = Resources.LoadZipFilter;
				okienko.InitialDirectory = PathToMC;
				switch (_where)
				{
					case "src":
						okienko.Title = Resources.LoadZipSRC;
						break;
					case "dst":
						okienko.Title = Resources.LoadZipDST;
						break;
				}


				if (okienko.ShowDialog() == DialogResult.OK)
				{
					wynik = true;
					switch (where)
					{
						case "src":
							SelectedFileNameSRC = okienko.SafeFileName;
							break;
						case "dst":
							SelectedFileNameDST = okienko.SafeFileName;
							break;
					}

					Show();
					_zip = ZipFile.Read(okienko.FileName);
					_progressBar1.Maximum = _zip.Count;
					_progressBar1.Value = 0;
					switch (_where)
					{
						case "src":
							_bwDepacking.RunWorkerAsync(TempSRC);
							Pathtoreturn = TempSRC;
							break;
						case "dst":
							_bwDepacking.RunWorkerAsync(TempDST);
							Pathtoreturn = TempDST;
							break;
					}
				}
			}

			// a w petli czekamy na koniec pracy bg workera
			while (_bwDepacking.IsBusy)
			{
				Application.DoEvents();
			}
			return wynik;
		}

		/// <summary>
		/// Metoda wczytuje plik graficzny.
		/// </summary>
		/// <returns>zwraca wczytany plik w formacie Bitmap</returns>
		public Bitmap LoadFile(string _pathToFile)
		{
			Bitmap loaded;

			using (var fs = new FileStream(_pathToFile, FileMode.Open, FileAccess.Read))
			{
				loaded = new Bitmap(Image.FromStream(fs));
			}

			return loaded;
		}


		/// <summary>
		/// Metoda porwnuje listy i kasuje z nich nie istniejace pathe do plikow
		/// Robi to dla obu list. Metoda wykorzystuje pomocnicze metody
		/// </summary>
		/// <param name="PathesListSRC"></param>
		/// <param name="PathesListDST"></param>
		public void PorownajListy(ref List<string> PathesListSRC, ref List<string> PathesListDST)
		{
			if (PathesListSRC.Count != 0 && PathesListDST.Count != 0)
			{
				PathesListSRC.Sort();
				PathesListDST.Sort();

				var listA = new List<string>(PathesListSRC);
				var listB = new List<string>(PathesListDST);

				listA = ReplaceInLists(PathesListSRC, "src", "...");
				listB = ReplaceInLists(PathesListDST, "dst", "...");

				listA = DifferencesInLists(listA, listB);
				listB = DifferencesInLists(listB, listA);

				PathesListSRC = ReplaceInLists(listA, "...", "src");
				PathesListDST = ReplaceInLists(listB, "...", "dst");
			}
		}

		/// <summary>
		/// Metoda wczytująca plik zip. W  niej czekamy za koniec pracy bg workera
		/// </summary>
		/// <summary>
		/// metoda przygotowujaca katalogi dla programu
		/// </summary>
		public void PrepareDirs(string what)
		{
			switch (what)
			{
				case "all":
					if (!Directory.Exists(TempTpm))
					{
						Directory.CreateDirectory(TempTpm);
						Directory.CreateDirectory(TempSRC);
						Directory.CreateDirectory(TempDST);
					}
					else
					{
						Directory.Delete(TempDST, true);
						Directory.Delete(TempSRC, true);
						Directory.Delete(TempTpm, true);
						Directory.CreateDirectory(TempTpm);
						Directory.CreateDirectory(TempSRC);
						Directory.CreateDirectory(TempDST);
					}
					break;
				case "src":
					Directory.Delete(TempSRC, true);
					Directory.CreateDirectory(TempSRC);
					break;
				case "dst":
					Directory.Delete(TempDST, true);
					Directory.CreateDirectory(TempDST);
					break;
			}
		}


		public void SaveFile(string what, Bitmap incoming_img)
		{
			Bitmap temp = incoming_img;
			string filename = what;
			using (var okienko = new SaveFileDialog())
			{
				okienko.Filter = ".png File|.png";
				okienko.FileName = what;
				if (okienko.ShowDialog() == DialogResult.OK)
				{
					temp.Save(okienko.FileName, ImageFormat.Png);
				}
			}
		}

		public void SaveZip()
		{
			var okienko = new SaveFileDialog
			              	{
			              		AddExtension = true,
			              		DefaultExt = ".zip",
			              		InitialDirectory = PathToMC,
			              		Filter = ".zip Archive|*.zip|All Files|*.*"
			              	};
			if (!Directory.EnumerateDirectories(TempDST).Any())
			{
				MessageBox.Show("No archive to save. Load some first !", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			okienko.FileName = SelectedFileNameDST;

			AddInfo("Yay, TexturePack edited with TPM"); //add info to pack.txt file

			if (okienko.ShowDialog() == DialogResult.OK)
			{
				SavePath = okienko.FileName;
				IEnumerable<string> _path1 = Directory.EnumerateDirectories(TempDST);
				IEnumerable<string> _path2 = Directory.EnumerateFiles(TempDST);
				_progressBar1.Maximum = _path1.Count() + _path2.Count();
				_progressBar1.Value = 0;
				Show();
				_bwPack.RunWorkerAsync();
			}
			while (_bwPack.IsBusy)
			{
				Application.DoEvents();
			}
		}

		private void AddInfo(string addLine)
		{
			string pathTo = TempDST + @"\pack.txt";
			string line;
			using (var reader = new StreamReader(pathTo))
			{
				line = reader.ReadLine();
				reader.Close();
			}

			var lines = new List<string> {line, addLine};

			File.Delete(pathTo);
			File.AppendAllLines(pathTo, lines);
		}

		/// <summary>
		/// Differenceses the in lists.
		/// </summary>
		/// <param name="lista1">Lista w której szukamy roznicy.</param>
		/// <param name="lista2">Lista z która porównujemy liste1 .</param>
		/// <returns>Zwraca liste1 z usunietymi roznicami</returns>
		private List<string> DifferencesInLists(List<string> lista1, List<string> lista2)
		{
			var returnList = new List<string>(lista1);
			IEnumerable<string> difference = lista1.Except(lista2);
			foreach (string item in difference)
			{
				returnList.Remove(item);
			}
			return returnList;
		}


		/// <summary>
		/// Replaces the string in lists.
		/// </summary>
		/// <param name="incomingList">The incoming list.</param>
		/// <param name="whatToFind">The what to find.</param>
		/// <param name="replaceTo">The replace to.</param>
		/// <returns></returns>
		private List<string> ReplaceInLists(List<string> incomingList, string whatToFind, string replaceTo)
		{
			return incomingList.Select(item => item.Replace(whatToFind, replaceTo)).ToList();
		}
		
		/// <summary>
		/// Scales the picture box.
		/// </summary>
		/// <param name="size">The size.</param>
		/// <returns>Scaled size</returns>
		public Size ScalePictureBox( Size size )  //todo skonczyc skalowanie
		{
			double x;
			double y;

			string box = null;
			if (size.Width > size.Height) box = "landscape";
			if (size.Width < size.Height) box = "portable";
			if (size.Width == size.Height) box = "square";
			switch (box)
			{
				case "landscape":
					
				case "portable":
				case "square":
					x = 400;
					y = 400;
					break;
			}

			x = 400;
			y = 400;

			return new Size((int)x,(int)y);
		}

		protected double scale;
		

		public Image LoadThumbnail(string what)
		{
			string path = " ";
			if (what == "src") path = TempSRC;
			if (what == "dst") path = TempDST;
			path += "pack.png";
			var fs = new FileStream(path, FileMode.Open, FileAccess.Read);
			Image thumbnail = Image.FromStream(fs);
			fs.Dispose();

			return thumbnail;
		}

		#region BW depaking zipa

		private void button1_Click(object sender, EventArgs e)
		{
			_bwDepacking.CancelAsync();
		}

		/// <summary>
		/// Metoda uruchamiajaca backgroudWorkera i depakujaca .zip do katalogu przekazanego przez e
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void bwDepacking_DoWork(object sender, DoWorkEventArgs e)
		{
			string _patch = e.Argument.ToString();
			int _progres = 0;
			foreach (ZipEntry files in _zip)
			{
				if (_bwDepacking.CancellationPending)
				{
					e.Cancel = true;
					break;
				}
				files.Extract(_patch);
				_bwDepacking.ReportProgress(_progres);
				_progres++;
			}
		}

		private void bwDepacking_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			_progressBar1.Value++;
		}

		private void bwDepacking_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			Hide();
		}

		#endregion BW depaking zipa

		#region BW Pack zipa

		private void bwPack_DoWork(object sender, DoWorkEventArgs e)
		{
			Debug.WriteLine("BW Pack just started");
			using (var zip = new ZipFile())
			{
				string _path = TempDST;


				zip.Comment = "Texture Pack mixed in Texturizer " + DateTime.Now.ToString("G");
				zip.CompressionMethod = CompressionMethod.Deflate;
				int _progres = 0;
				// add dirs
				IEnumerable<string> ToZip = Directory.EnumerateDirectories(_path);
				foreach (string item in ToZip)
				{
					string _zipPath = item.Substring(item.LastIndexOf((@"\")));
					//kasuje poczatek patha zostawiajac tylko koncowke dla utworzenia katalogu w zip
					zip.AddItem(item, _zipPath);

					if (_bwPack.CancellationPending)
					{
						e.Cancel = true;
						break;
					}

					_bwPack.ReportProgress(_progres);
					_progres++;
					Debug.Write(_progres.ToString() + "...");
					//Thread.Sleep(500);
				}

				//add files
				ToZip = Directory.EnumerateFiles(_path);
				foreach (string item in ToZip)
				{
					zip.AddItem(item, "");
					if (_bwPack.CancellationPending)
					{
						e.Cancel = true;
						break;
					}
					_bwPack.ReportProgress(_progres);
					_progres++;
				}

				zip.Save(SavePath);
				Debug.WriteLine("BW Pack just ended");
			}
		}

		private void bwPack_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			_progressBar1.Value = e.ProgressPercentage;
		}

		#endregion

		#region InitializeComponent
		

		

		#endregion

		public void DSTImageSave(string dstPath, Bitmap dstImage)
		{
			if (dstImage != null)
			{
				dstImage.Save(dstPath, ImageFormat.Png);	
			}
			
		}
	}
}