﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.IO.Pipes;
using System.Linq;
using System.Windows.Forms;
using System.Xml;
using Ionic.Zip;
using Ionic.Zlib;
using System.Threading;

namespace Texturizer
{
	/// <summary>
	/// IO Class dla dzialan na plikach oraz metody pomocnicze
	/// </summary>
	public class IOClass : Form
	{
		#region PROP
		//zmienna sluzaca do zliczenia wszystkich dostepnych plikow graficznych
		public int FoundFiles { get; set; }

		private string Patchtmp { get; set; }

		private string Pathtoreturn { get; set; }

		private string TempDST { get; set; }

		private string TempSRC { get; set; }
		private readonly string PathToMC = @"%appdata%.minecraft\texturepacks";

		// trzy zmienne do okreslenia sciezek dostepu dla katalogu do grafik programu
		private string TempTpm { get; set; }



		// String do okreslenia path-a dla pliku konfiguracji, tymczasowo umieszczony na stale w katalogu C:
		private string _pathtoconfig = @"PathesToFiles.xml";
		private string SavePath { get; set; }

		private IEnumerable<string> FilesToZip;

		private Button button1;
		private System.ComponentModel.BackgroundWorker bwDepacking;
		private Label _label1;
		private Form _orginalform;
		private ProgressBar _progressBar1;
		private ZipFile _zip;
		private string _selectedFileNameSRC;
		private System.ComponentModel.BackgroundWorker bwPack;
	
		public string SelectedFileNameSRC
		{
			get
			{
				if (_selectedFileNameSRC==null)
				{
					return "NO IMAGE";
				}
				return _selectedFileNameSRC;
			}
			set { _selectedFileNameSRC = value; }
		}

		private string _selectedFileNameDST;
		public string SelectedFileNameDST
		{
			get
			{
				if (_selectedFileNameDST==null)
				{
					return "NO IMAGE";
				}
				return _selectedFileNameDST;
			}
			set { _selectedFileNameDST = value; }
		}

		#endregion
		

		/// <summary>
		/// Konstruktor klasy IO
		/// </summary>
		/// <param name="sendedform"></param>
		public IOClass( Form sendedform )
		{
			InitializeComponent();
			_orginalform = sendedform;
			TempTpm = Path.GetTempPath() + "TMP";
			TempSRC = TempTpm + "\\src\\";
			TempDST = TempTpm + "\\dst\\";
		}


		/// <summary>
		/// Metoda odczytuyje jakie powinny byc pliki z "PathesToFiles.xml i sprawdza czy takie
		/// pliki istnieja na dysku w katalogu temp/tmp/src i dst
		/// </summary>
		/// <param name="_patches">przekazana tablica jako REF</param>
		/// <param name="_incomingwhat">source czy destination </param>
		public void CzySaTakiePliki( ref List<string> PathesListSRC, ref List<string> PathesListDST, string _incomingwhat )
		{
			FoundFiles = 0;
			var _xml = new XmlDocument();
			_xml.Load(_pathtoconfig);
			var _node = _xml.GetElementsByTagName("TPM");				//szukaj childnody dla glowej galezi TPM
			int _childnodes = _node.Item(0).ChildNodes.Count;                  //odczyt ilosci childnodow dla glownej galezi
			if (_incomingwhat == "src")									//czysci tablicze src
			{
				PathesListSRC.Clear();
			}
			if (_incomingwhat == "dst")									// czysci tablice dst
			{
				PathesListDST.Clear();
			}
			for (int i = 0; i < _childnodes; i++)					//MAIN LOOP, petla dla znalezionych nodow
			{
				if (_incomingwhat == "src")
				{
					Patchtmp = TempSRC + _node.Item(0).ChildNodes.Item(i).Attributes.GetNamedItem("path").InnerText; ;
				}
				if (_incomingwhat == "dst")
				{
					Patchtmp = TempDST + _node.Item(0).ChildNodes.Item(i).Attributes.GetNamedItem("path").InnerText; ;
				}

				for (int a = 0; a < _node.Item(0).ChildNodes.Item(i).ChildNodes.Count; a++) //petla dla tagow noda
				{
					var _file = _node.Item(0).ChildNodes.Item(i).ChildNodes.Item(a).InnerText;			//pobiera jaki plik z xml-a
					var _fullpath = Patchtmp + _file;														//skleja sciezke z plikiem
					if (File.Exists(_fullpath))
					{
						if (_incomingwhat == "src")
						{
							PathesListSRC.Add(_fullpath);
						}
						if (_incomingwhat == "dst")
						{
							PathesListDST.Add(_fullpath);
						}
						FoundFiles++;
					}
				}
			}
		}

		public void LoadZip( string where )
		{
			string _where = where;
			PrepareDirs(_where);
			

			using (var okienko = new OpenFileDialog())
			{
				okienko.Filter = ".zip Archive|*.zip| All Files|*.*";
				okienko.InitialDirectory = PathToMC;
				switch (_where)
				{
					case "src":okienko.Title = "Load Source .zip archive";break;
					case "dst":okienko.Title = "Load Destination .zip archive";break;
				}
				
				

				if (okienko.ShowDialog() == DialogResult.OK)
				{
					switch (where)
					{
						case "src":SelectedFileNameSRC = okienko.SafeFileName;
							break;
						case "dst": SelectedFileNameDST = okienko.SafeFileName;
							break;
					}

					this.Show();
					_zip = ZipFile.Read(okienko.FileName);
					_progressBar1.Maximum = _zip.Count;
					_progressBar1.Value = 0;
					switch (_where)
					{
						case "src": bwDepacking.RunWorkerAsync(TempSRC); Pathtoreturn = TempSRC; break;
						case "dst": bwDepacking.RunWorkerAsync(TempDST); Pathtoreturn = TempDST; break;
					}
				}
			}

			// a w petli czekamy na koniec pracy bg workera
			while (bwDepacking.IsBusy)
			{
				Application.DoEvents();
			}
		}

		/// <summary>
		/// Metoda wczytuje plik graficzny.
		/// </summary>
		/// <returns>zwraca wczytany plik w formacie Bitmap</returns>
		public Bitmap LoadFile( string _pathToFile )
		{
			//Bitmap loaded = new Bitmap(global::Texturizer.Properties.Resources.PicBox_Background);
			//if (_pathToFile==null)
			//{
			//    return loaded;
			//}

			Bitmap loaded;

			using (var fs = new FileStream(_pathToFile, FileMode.Open, FileAccess.Read))
			{
				loaded = new Bitmap(Bitmap.FromStream(fs));
			}

			return loaded;
		}


		/// <summary>
		/// Metoda porwnuje listy i kasuje z nich nie istniejace pathe do plikow
		/// Robi to dla obu list. Metoda wykorzystuje pomocnicze metody
		/// </summary>
		/// <param name="PathesListSRC"></param>
		/// <param name="PathesListDST"></param>
		public void PorownajListy( ref List<string> PathesListSRC, ref List<string> PathesListDST )
		{
			if (PathesListSRC.Count != 0 && PathesListDST.Count != 0)
			{
				PathesListSRC.Sort();
				PathesListDST.Sort();

				var listA = new List<string>(PathesListSRC);
				var listB = new List<string>(PathesListDST);

				listA = ReplaceInLists(PathesListSRC, "src", "...");
				listB = ReplaceInLists(PathesListDST, "dst", "...");

				listA = DifferencesInLists(listA, listB);
				listB = DifferencesInLists(listB, listA);

				PathesListSRC = ReplaceInLists(listA, "...", "src");
				PathesListDST = ReplaceInLists(listA, "...", "dst");
			}
		}

		/// <summary>
		/// Metoda wczytująca plik zip. W  niej czekamy za koniec pracy bg workera
		/// </summary>
		/// <summary>
		/// metoda przygotowujaca katalogi dla programu
		/// </summary>
		public void PrepareDirs( string what )
		{
			switch (what)
			{
				case "all":
					if (!Directory.Exists(TempTpm))
					{
						Directory.CreateDirectory(TempTpm);
						Directory.CreateDirectory(TempSRC);
						Directory.CreateDirectory(TempDST);
					}
					else
					{
						Directory.Delete(TempDST, true);
						Directory.Delete(TempSRC, true);
						Directory.Delete(TempTpm, true);
						Directory.CreateDirectory(TempTpm);
						Directory.CreateDirectory(TempSRC);
						Directory.CreateDirectory(TempDST);
					}
					break;
				case "src":
					Directory.Delete(TempSRC, true);
					Directory.CreateDirectory(TempSRC);
					break;
				case "dst":
					Directory.Delete(TempDST, true);
					Directory.CreateDirectory(TempDST);
					break;
			}
		}

		
		public void SaveFile( string what, Bitmap incoming_img )
		{
			Bitmap temp = incoming_img;
			string filename = what;
			using (SaveFileDialog okienko = new SaveFileDialog())
			{
				okienko.Filter = ".png File|.png";
				okienko.FileName = what;
				if (okienko.ShowDialog() == DialogResult.OK)
				{
					temp.Save(okienko.FileName, ImageFormat.Png);
				}
			}
		}

		public void SaveZip()
		{
			
			var okienko = new SaveFileDialog();
			okienko.AddExtension = true;
			okienko.DefaultExt = ".zip";
			okienko.InitialDirectory = PathToMC ;
			okienko.Filter = ".zip Archive|*.zip|All Files|*.*";
			if (!Directory.EnumerateDirectories(TempDST).Any())
			{
				MessageBox.Show("No archive to save. Load some first !", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			okienko.FileName = SelectedFileNameDST;

			AddInfo("Yay, TexturePack edited with TPM");							//add info to pack.txt file

			if (okienko.ShowDialog() == DialogResult.OK)
			{
				SavePath = okienko.FileName;
				var _path1 = Directory.EnumerateDirectories(TempDST);
				var _path2 = Directory.EnumerateFiles(TempDST);
				_progressBar1.Maximum = _path1.Count()+_path2.Count();
				_progressBar1.Value = 0;
				this.Show();
				bwPack.RunWorkerAsync();
			}
			while (bwPack.IsBusy)
			{
				Application.DoEvents();
			}
			
		}

		private void AddInfo(string addLine)
		{
			string pathTo = TempDST+@"\pack.txt";
			string line;
			using (var reader = new StreamReader(pathTo))
			{
				line=reader.ReadLine();
				reader.Close();
			}
			
			List<string> lines = new List<string>();

			lines.Add(line);
			lines.Add(addLine);

			File.Delete(pathTo);
			File.AppendAllLines(pathTo,lines);
			
		}

		/// <summary>
		/// Differenceses the in lists.
		/// </summary>
		/// <param name="lista1">Lista w której szukamy roznicy.</param>
		/// <param name="lista2">Lista z która porównujemy liste1 .</param>
		/// <returns>Zwraca liste1 z usunietymi roznicami</returns>
		private List<string> DifferencesInLists( List<string> lista1 , List<string> lista2 )
		{
			List<string> returnList = new List<string>(lista1);
			var difference = lista1.Except(lista2);
			foreach (var item in difference)
			{
				returnList.Remove(item);
			}
			return returnList;
		}



		/// <summary>
		/// Replaces the string in lists.
		/// </summary>
		/// <param name="incomingList">The incoming list.</param>
		/// <param name="whatToFind">The what to find.</param>
		/// <param name="replaceTo">The replace to.</param>
		/// <returns></returns>
		private List<string> ReplaceInLists( List<string> incomingList, string whatToFind, string replaceTo )
		{
			var temp = new List<string>();
			foreach (var item in incomingList)
			{
				temp.Add(item.Replace(whatToFind, replaceTo));
			}

			return temp;
		}

		#region BW depaking zipa

		private void button1_Click( object sender, EventArgs e )
		{
			bwDepacking.CancelAsync();
		}

		/// <summary>
		/// Metoda uruchamiajaca backgroudWorkera i depakujaca .zip do katalogu przekazanego przez e
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void bwDepacking_DoWork( object sender, System.ComponentModel.DoWorkEventArgs e )
		{
			string _patch = e.Argument.ToString();
			int _progres = 0;
			foreach (var files in _zip)
			{
				if (bwDepacking.CancellationPending == true){e.Cancel = true;break;}
				files.Extract(_patch);
				bwDepacking.ReportProgress(_progres);
				_progres++;
			}
			
		}

		private void bwDepacking_ProgressChanged( object sender, System.ComponentModel.ProgressChangedEventArgs e )
		{
			_progressBar1.Value++;
		}

		private void bwDepacking_RunWorkerCompleted( object sender, System.ComponentModel.RunWorkerCompletedEventArgs e )
		{
			this.Hide();
		}

		#endregion BW depaking zipa

		private void bwPack_DoWork( object sender, System.ComponentModel.DoWorkEventArgs e )
		{
			System.Diagnostics.Debug.WriteLine("BW Pack just started");
			using (var zip = new ZipFile())
			{
				string _path = TempDST;

				zip.Comment = "Texture Pack mixed in Texturizer " + System.DateTime.Now.ToString("G");
				zip.CompressionLevel = CompressionLevel.Default;
				zip.CompressionMethod = CompressionMethod.Deflate;
				int _progres = 0;
				// add dirs
				var ToZip = Directory.EnumerateDirectories(_path);
				foreach (var item in ToZip)
				{
					var _zipPath = item.Substring(item.LastIndexOf((@"\"))); //kasuje poczatek patha zostawiajac tylko koncowke dla utworzenia katalogu w zip
					zip.AddItem(item, _zipPath);

					if (bwPack.CancellationPending == true) { e.Cancel = true; break; }

					bwPack.ReportProgress(_progres);
					_progres++; System.Diagnostics.Debug.Write(_progres.ToString()+"...");
					//Thread.Sleep(500);
				}

				//add files
				ToZip = Directory.EnumerateFiles(_path);
				foreach (var item in ToZip)
				{
					zip.AddItem(item, "");
					if (bwPack.CancellationPending == true){e.Cancel = true; break;}
					bwPack.ReportProgress(_progres);
					_progres++;
				}

				zip.Save(SavePath);
				System.Diagnostics.Debug.WriteLine("BW Pack just ended");
			}
		}

		private void bwPack_ProgressChanged( object sender, System.ComponentModel.ProgressChangedEventArgs e )
		{
			_progressBar1.Value = e.ProgressPercentage;
		}
		


		private void InitializeComponent()
		{
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(IOClass));
			this.button1 = new System.Windows.Forms.Button();
			this._progressBar1 = new System.Windows.Forms.ProgressBar();
			this.bwDepacking = new System.ComponentModel.BackgroundWorker();
			this._label1 = new System.Windows.Forms.Label();
			this.bwPack = new System.ComponentModel.BackgroundWorker();
			this.SuspendLayout();
			// 
			// button1
			// 
			this.button1.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.button1.Location = new System.Drawing.Point(169, 68);
			this.button1.Name = "button1";
			this.button1.Size = new System.Drawing.Size(75, 23);
			this.button1.TabIndex = 0;
			this.button1.Text = "Cancel";
			this.button1.UseVisualStyleBackColor = true;
			this.button1.Click += new System.EventHandler(this.button1_Click);
			// 
			// _progressBar1
			// 
			this._progressBar1.Location = new System.Drawing.Point(82, 39);
			this._progressBar1.Name = "_progressBar1";
			this._progressBar1.Size = new System.Drawing.Size(264, 23);
			this._progressBar1.TabIndex = 1;
			// 
			// bwDepacking
			// 
			this.bwDepacking.WorkerReportsProgress = true;
			this.bwDepacking.WorkerSupportsCancellation = true;
			this.bwDepacking.DoWork += new System.ComponentModel.DoWorkEventHandler(this.bwDepacking_DoWork);
			this.bwDepacking.ProgressChanged += new System.ComponentModel.ProgressChangedEventHandler(this.bwDepacking_ProgressChanged);
			this.bwDepacking.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(this.bwDepacking_RunWorkerCompleted);
			// 
			// _label1
			// 
			this._label1.AutoSize = true;
			this._label1.Font = new System.Drawing.Font("Tahoma", 18F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(238)));
			this._label1.Location = new System.Drawing.Point(155, 9);
			this._label1.Name = "_label1";
			this._label1.Size = new System.Drawing.Size(128, 29);
			this._label1.TabIndex = 2;
			this._label1.Text = " Wait . . . ";
			this._label1.TextAlign = System.Drawing.ContentAlignment.TopCenter;
			// 
			// bwPack
			// 
			this.bwPack.WorkerReportsProgress = true;
			this.bwPack.WorkerSupportsCancellation = true;
			this.bwPack.DoWork += new System.ComponentModel.DoWorkEventHandler(this.bwPack_DoWork);
			this.bwPack.ProgressChanged += new System.ComponentModel.ProgressChangedEventHandler(this.bwPack_ProgressChanged);
			this.bwPack.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(this.bwDepacking_RunWorkerCompleted);
			// 
			// IOClass
			// 
			this.CancelButton = this.button1;
			this.ClientSize = new System.Drawing.Size(423, 99);
			this.Controls.Add(this._label1);
			this.Controls.Add(this._progressBar1);
			this.Controls.Add(this.button1);
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "IOClass";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Texturizer";
			this.TopMost = true;
			this.ResumeLayout(false);
			this.PerformLayout();

		}


	}


}